"""
Version: 0.1.2: Fixed MANIFEST! Missing assets
Version: 0.1.1: Refactored .filter to .filters to prevent namespace collision
Version: 0.1.0: First release! Added subpixel resolution to prior work.
"""

__version__ = "0.1.2"
