from .colors import NamedColors
from .palettes import ColorLoversPalette

from .interpolation import RGBa_interpolation
from .interpolation import LABa_interpolation

from .gradient import linear_gradient

matplotlib_colors = NamedColors()
