from .artist import Artist
from .canvas import Canvas
from .animation import Animation

from .primitives import circle, line, ellipse, rectangle, polyline, text

from .color import linear_gradient, ColorLoversPalette
from .io import canvas2gif, canvas2mp4

from . import transform
from . import motion
from . import filters
from . import color
